@@ -1,11 +1,11 @@
 from openfisca_core.errors import SituationParsingError
 
 
-def calculate_output_add(simulation, variable_name, period):
+def calculate_output_add(simulation, variable_name: str, period):
     return simulation.calculate_add(variable_name, period)
 
 
-def calculate_output_divide(simulation, variable_name, period):
+def calculate_output_divide(simulation, variable_name: str, period):
     return simulation.calculate_divide(variable_name, period)
 
 
