@@ -1,12 +1,25 @@
 """Data types and protocols used by OpenFisca Core.
 
-The type definitions included in this sub-package are intented for
+The type definitions included in this sub-package are intended for
 contributors, to help them better understand and document contracts
 and expected behaviours.
 
 Official Public API:
+    * :attr:`.Array`
     * ``ArrayLike``
-    * :attr:`.ArrayType`
+    * :attr:`.Cache`
+    * :attr:`.Entity`
+    * :attr:`.Formula`
+    * :attr:`.Holder`
+    * :attr:`.Instant`
+    * :attr:`.ParameterNodeAtInstant`
+    * :attr:`.Params`
+    * :attr:`.Period`
+    * :attr:`.Population`
+    * :attr:`.Role`,
+    * :attr:`.Simulation`,
+    * :attr:`.TaxBenefitSystem`
+    * :attr:`.Variable`
 
 Note:
     How imports are being used today::
@@ -14,7 +27,6 @@ Note:
         from openfisca_core.types import *  # Bad
         from openfisca_core.types.data_types.arrays import ArrayLike  # Bad
 
-
     The previous examples provoke cyclic dependency problems, that prevents us
     from modularizing the different components of the library, so as to make
     them easier to test and to maintain.
@@ -37,9 +49,39 @@ Note:
 
 # Official Public API
 
-from .data_types import (  # noqa: F401
+from ._data import (  # noqa: F401
+    Array,
     ArrayLike,
-    ArrayType,
     )
 
-__all__ = ["ArrayLike", "ArrayType"]
+from ._domain import (  # noqa: F401
+    Entity,
+    Formula,
+    Holder,
+    Instant,
+    ParameterNodeAtInstant,
+    Params,
+    Period,
+    Population,
+    Role,
+    Simulation,
+    TaxBenefitSystem,
+    Variable,
+    )
+
+__all__ = [
+    "Array",
+    "ArrayLike",
+    "Entity",
+    "Formula",
+    "Holder",
+    "Instant",
+    "ParameterNodeAtInstant",
+    "Params",
+    "Period",
+    "Population",
+    "Role",
+    "Simulation",
+    "TaxBenefitSystem",
+    "Variable",
+    ]
